/***************************************************************************
 *   Copyright (C) 2007 by Sébastien Laoût                                 *
 *   slaout@linux62.org                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef THEMEMANAGER_H
#define THEMEMANAGER_H

#include <qobject.h>
#include <qstring.h>
#include <qcolor.h>
#include <qptrlist.h>

class QPixmap;
class QWidget;

/**
 * @author Sébastien Laoût <slaout@linux62.org>
 */
class Theme
{
  public:
	// Constructor:
	Theme(const QString &location, const QString &folderName);
	~Theme();

	// Theme informations:
	QString location() const;
	QString folderName() const;
	QString themeName() const;
	// Colors:
	QColor  backgroundColor() const;
	int     backgroundColorOpacity() const;
	QColor  textColor() const;
	QColor  nextPlayingColor() const;
	QColor  dateColor() const;
	QColor  hourColor() const;
	QColor  progressBackgroundColor() const;
	QColor  progressBarColor() const;
	QColor  progressBackgroundTextColor() const;
	QColor  progressBarTextColor() const;
	// Author:
	QString authorName() const;
	QString authorEMail() const;
	QString authorURL() const;
	QString copyrightInformation() const;
	bool    authorLocked() const;

	// Pixmap:
	const QPixmap &backgroundPixmap(int width, int height);
	const QPixmap &previewPixmap(int width, int height);
	void setUsed(bool used);

	// For the editor to store a temporar version:
	void copyTo(Theme *theme) const;

	static Theme *current();

	// GET:
	// Colors:
	QColor  backgroundColorSetting() const;
	QColor  textColorSetting() const;
	QColor  nextPlayingColorSetting() const;
	QColor  dateColorSetting() const;
	QColor  hourColorSetting() const;
	QColor  progressBackgroundColorSetting() const;
	QColor  progressBarColorSetting() const;
	QColor  progressBackgroundTextColorSetting() const;
	QColor  progressBarTextColorSetting() const;

	// DEFAULT COLORS:
	QColor defaultBackgroundColor() const;
	QColor defaultTextColor() const;
	QColor defaultNextPlayingColor() const;
	QColor defaultDateColor() const;
	QColor defaultHourColor() const;
	QColor defaultProgressBackgroundColor() const;
	QColor defaultProgressBarColor() const;
	QColor defaultProgressBackgroundTextColor() const;
	QColor defaultProgressBarTextColor() const;

	// SET:
	// Theme informations:
	void setThemeName(const QString &name);
	// Colors:
	void setBackgroundColor(const QColor &color);
	void setBackgroundColorOpacity(int opacity);
	void setTextColor(const QColor &color);
	void setNextPlayingColor(const QColor &color);
	void setDateColor(const QColor &color);
	void setHourColor(const QColor &color);
	void setProgressBackgroundColor(const QColor &color);
	void setProgressBarColor(const QColor &color);
	void setProgressBackgroundTextColor(const QColor &color);
	void setProgressBarTextColor(const QColor &color);
	// Author:
	void setAuthorName(const QString &name);
	void setAuthorEMail(const QString &eMail);
	void setAuthorURL(const QString &url);
	void setCopyrightInformation(const QString &information);
	void setAuthorLocked(bool lock);

	// Disk Management:
	void load();
	void save();

  protected:
	QPixmap *computePixmap(int width, int height, const QStringList &locations) const;

	// Tools:
	static bool tooDark(const QColor &color);
	static QColor mixColors(const QColor &color1, const QColor &color2);
	static bool isWideScreen(int width, int height);

  private:
	// Theme informations:
	QString m_location;
	QString m_folderName;
	QString m_themeName;
	// Colors:
	QColor  m_backgroundColor;
	int     m_backgroundColorOpacity;
	QColor  m_textColor;
	QColor  m_nextPlayingColor;
	QColor  m_dateColor;
	QColor  m_hourColor;
	QColor  m_progressBackgroundColor;
	QColor  m_progressBarColor;
	QColor  m_progressBackgroundTextColor;
	QColor  m_progressBarTextColor;
	// Author:
	QString m_authorName;
	QString m_authorEMail;
	QString m_authorURL;
	QString m_copyrightInformation;
	bool    m_authorLocked;

	QPixmap *m_backgroundPixmap;
	QPixmap *m_previewPixmap;
};

class ThemeList : public QPtrList<Theme>
{
  public:
	ThemeList();
  protected:
	int compareItems(QPtrCollection::Item item1, QPtrCollection::Item item2);
};

//bool operator<(const Theme &theme1, const Theme &theme2);

/**
 * @author Sébastien Laoût <slaout@linux62.org>
 */
class ThemeManager : public QObject
{
  Q_OBJECT
  private:
	ThemeManager();
	~ThemeManager();

	void loadThemeList();

  signals:
	void themeChanged();

  public:
	Theme *currentTheme();

	static ThemeManager *instance();
	static void chooseTheme(int screenWidth, int screenHeight, QWidget *parent);

	const ThemeList &themes() const;

	void setTheme(Theme *theme);

  private:
	ThemeList  m_themes;
	Theme     *m_currentTheme;

	static ThemeManager *s_instance;
};

#endif // THEMEMANAGER_H
